@@ -38,6 +38,7 @@
 
 #include "cpu_registers.h"
 #include "cpuinfo.h"
+#include "hw_monitoring.h"
 #include "log.h"
 #include "machine.h"
 #include "uncore_monitoring.h"
@@ -77,6 +78,85 @@ get_cache_info(const struct pqos_cacheinfo *cache_info,
         return PQOS_RETVAL_OK;
 }
 
+/**
+ * @brief Checks L3 feature enable status across all CPU sockets
+ *
+ * It also validates if L3 feature enabling is consistent across CPU sockets.
+ * At the moment, such scenario is considered as error that requires CAT reset.
+ *
+ * @param [in] cpu detected CPU topology
+ * @param [in] feature_name printable feature name
+ * @param [in] reg configuration register address
+ * @param [in] bit configuration bit
+ * @param [out] enabled place to store L3 feature enabling status
+ *
+ * @return Operations status
+ * @retval PQOS_RETVAL_OK on success
+ */
+static int
+hw_cap_l3_feature(const struct pqos_cpuinfo *cpu,
+                  const char *feature_name,
+                  uint32_t reg,
+                  uint64_t bit,
+                  int *enabled)
+{
+        unsigned *l3cat_ids = NULL;
+        unsigned l3cat_id_num = 0;
+        unsigned enabled_num = 0, disabled_num = 0;
+        unsigned j;
+        int ret = PQOS_RETVAL_OK;
+
+        if (enabled == NULL || cpu == NULL)
+                return PQOS_RETVAL_PARAM;
+
+        /**
+         * Get list of l3cat id's
+         */
+        l3cat_ids = pqos_cpu_get_l3cat_ids(cpu, &l3cat_id_num);
+        if (l3cat_ids == NULL)
+                return PQOS_RETVAL_RESOURCE;
+
+        for (j = 0; j < l3cat_id_num; j++) {
+                uint64_t value = 0;
+                unsigned core = 0;
+
+                ret = pqos_cpu_get_one_by_l3cat_id(cpu, l3cat_ids[j], &core);
+                if (ret != PQOS_RETVAL_OK)
+                        goto hw_cap_l3_feature_exit;
+
+                if (msr_read(core, reg, &value) != MACHINE_RETVAL_OK) {
+                        ret = PQOS_RETVAL_ERROR;
+                        goto hw_cap_l3_feature_exit;
+                }
+
+                if (value & bit)
+                        enabled_num++;
+                else
+                        disabled_num++;
+        }
+
+        if (disabled_num > 0 && enabled_num > 0) {
+                LOG_ERROR("Inconsistent %s settings across l3cat_ids."
+                          "Please reset CAT or reboot your system!\n",
+                          feature_name);
+                ret = PQOS_RETVAL_ERROR;
+
+        } else if (enabled_num > 0)
+                *enabled = 1;
+        else
+                *enabled = 0;
+
+        if (ret == PQOS_RETVAL_OK)
+                LOG_INFO("%s is %s\n", feature_name,
+                         (*enabled) ? "enabled" : "disabled");
+
+hw_cap_l3_feature_exit:
+        if (l3cat_ids != NULL)
+                free(l3cat_ids);
+
+        return ret;
+}
+
 /**
  * @brief Adds new event type to \a mon monitoring structure
  *
@@ -88,6 +168,7 @@ get_cache_info(const struct pqos_cacheinfo *cache_info,
  * @param scale_factor event specific scale factor
  * @param counter_length counter bit length for the event
  * @param max_num_events maximum number of events that \a mon can accommodate
+ * @param iordt I/O RDT support
  */
 static void
 add_monitoring_event(struct pqos_cap_mon *mon,
@@ -96,7 +177,8 @@ add_monitoring_event(struct pqos_cap_mon *mon,
                      const unsigned max_rmid,
                      const uint32_t scale_factor,
                      const unsigned counter_length,
-                     const unsigned max_num_events)
+                     const unsigned max_num_events,
+                     const int iordt)
 {
         if (mon->num_events >= max_num_events) {
                 LOG_WARN("%s() no space for event type %d (resource id %u)!\n",
@@ -105,14 +187,105 @@ add_monitoring_event(struct pqos_cap_mon *mon,
         }
 
         LOG_DEBUG("Adding monitoring event: resource ID %u, "
-                  "type %x to table index %u\n",
-                  res_id, event_type, mon->num_events);
+                  "type 0x%x, iordt %d\n",
+                  res_id, event_type, iordt);
 
         mon->events[mon->num_events].type = (enum pqos_mon_event)event_type;
         mon->events[mon->num_events].max_rmid = max_rmid;
         mon->events[mon->num_events].scale_factor = scale_factor;
         mon->events[mon->num_events].counter_length = counter_length;
+        mon->events[mon->num_events].iordt = iordt;
         mon->num_events++;
+
+        if (iordt)
+                mon->iordt = 1;
+}
+
+int
+hw_cap_mon_iordt(const struct pqos_cpuinfo *cpu, int *enabled)
+{
+        return hw_cap_l3_feature(cpu, "L3 I/O RDT monitoring",
+                                 PQOS_MSR_L3_IO_QOS_CFG,
+                                 PQOS_MSR_L3_IO_QOS_MON_EN, enabled);
+}
+
+/**
+ * @brief Discovers HW monitoring SNC support
+ *
+ * @param [in] cpu detected CPU topology
+ * @param [out] snc_num Number of monitoring clusters
+ * @param [out] snc_mode SNC monitoring mode
+ *
+ * @return Operation status
+ * @retval PQOS_RETVAL_OK success
+ */
+PQOS_STATIC int
+hw_cap_mon_snc_state(const struct pqos_cpuinfo *cpu,
+                     unsigned *snc_num,
+                     enum pqos_snc_mode *snc_mode)
+{
+        int numa_num = cpuinfo_get_numa_num(cpu);
+        int socket_num = cpuinfo_get_socket_num(cpu);
+        unsigned i, sock_count, *sockets = NULL;
+        int ret = PQOS_RETVAL_OK;
+        enum pqos_snc_mode st;
+        enum pqos_snc_mode mode = (enum pqos_snc_mode)(-1);
+
+        if (cpu == NULL || snc_num == NULL || snc_mode == NULL)
+                return PQOS_RETVAL_PARAM;
+
+        if (numa_num < 0 || socket_num < 0)
+                return PQOS_RETVAL_ERROR;
+        if (numa_num == 0 || socket_num == 0 || numa_num == socket_num) {
+                *snc_num = 1;
+                *snc_mode = PQOS_SNC_LOCAL;
+                return PQOS_RETVAL_OK;
+        }
+
+        sockets = pqos_cpu_get_sockets(cpu, &sock_count);
+        if (sockets == NULL || sock_count == 0) {
+                printf("Error retrieving information for Sockets\n");
+                ret = PQOS_RETVAL_ERROR;
+                goto hw_cap_mon_snc_state_exit;
+        }
+
+        for (i = 0; i < sock_count; i++) {
+                unsigned lcore;
+                uint64_t val = 0;
+                uint32_t reg = PQOS_MSR_SNC_CFG;
+
+                ret = pqos_cpu_get_one_core(cpu, sockets[i], &lcore);
+                if (ret != PQOS_RETVAL_OK) {
+                        printf("Error retrieving lcore for socket %u\n",
+                               sockets[i]);
+                        ret = PQOS_RETVAL_ERROR;
+                        break;
+                };
+
+                if (msr_read(lcore, reg, &val) != MACHINE_RETVAL_OK) {
+                        ret = PQOS_RETVAL_ERROR;
+                        break;
+                }
+
+                st = (val & 1) ? PQOS_SNC_LOCAL : PQOS_SNC_TOTAL;
+                if (mode == (enum pqos_snc_mode)(-1))
+                        mode = st;
+                else if (mode != st) {
+                        printf("Error inconsistent SNC mode\n");
+                        ret = PQOS_RETVAL_ERROR;
+                        break;
+                };
+        };
+
+        if (ret == PQOS_RETVAL_OK) {
+                *snc_num = numa_num / socket_num;
+                *snc_mode = mode;
+        }
+
+hw_cap_mon_snc_state_exit:
+        if (sockets)
+                free(sockets);
+        return ret;
 }
 
 /**
@@ -179,13 +352,14 @@ hw_cap_mon_discover(struct pqos_cap_mon **r_mon, const struct pqos_cpuinfo *cpu)
          */
         lcpuid(0xf, 1, &cpuid_0xf_1); /**< query resource monitoring */
 
-        if (cpuid_0xf_1.edx & 1)
+        if (cpuid_0xf_1.edx & PQOS_CPUID_MON_L3_OCCUP_BIT)
                 num_events++; /**< LLC occupancy */
-        if (cpuid_0xf_1.edx & 2)
+        if (cpuid_0xf_1.edx & PQOS_CPUID_MON_TMEM_BW_BIT)
                 num_events++; /**< total memory bandwidth event */
-        if (cpuid_0xf_1.edx & 4)
+        if (cpuid_0xf_1.edx & PQOS_CPUID_MON_LMEM_BW_BIT)
                 num_events++; /**< local memory bandwidth event */
-        if ((cpuid_0xf_1.edx & 2) && (cpuid_0xf_1.edx & 4))
+        if ((cpuid_0xf_1.edx & PQOS_CPUID_MON_TMEM_BW_BIT) &&
+            (cpuid_0xf_1.edx & PQOS_CPUID_MON_LMEM_BW_BIT))
                 num_events++; /**< remote memory bandwidth virtual event */
 
         ret = uncore_mon_discover(&uncore_events);
@@ -233,54 +407,88 @@ hw_cap_mon_discover(struct pqos_cap_mon **r_mon, const struct pqos_cpuinfo *cpu)
         mon->max_rmid = max_rmid;
         mon->l3_size = l3_size;
 
+        /* Detect SNC state */
+        ret = hw_cap_mon_snc_state(cpu, &mon->snc_num, &mon->snc_mode);
+        if (ret != PQOS_RETVAL_OK) {
+                LOG_ERROR("Error reading SNC information!\n");
+                free(mon);
+                return PQOS_RETVAL_ERROR;
+        }
+
         {
                 const unsigned max_rmid = cpuid_0xf_1.ecx + 1;
                 const uint32_t scale_factor = cpuid_0xf_1.ebx;
                 const unsigned counter_length = (cpuid_0xf_1.eax & 0x7f) + 24;
 
-                if (cpuid_0xf_1.edx & 1)
+                if (cpuid_0xf_1.edx & PQOS_CPUID_MON_L3_OCCUP_BIT) {
+                        int iordt =
+                            cpuid_0xf_1.eax & PQOS_CPUID_MON_IO_OCCUP_BIT ? 1
+                                                                          : 0;
+
                         add_monitoring_event(mon, 1, PQOS_MON_EVENT_L3_OCCUP,
                                              max_rmid, scale_factor,
-                                             counter_length, num_events);
-                if (cpuid_0xf_1.edx & 2)
+                                             counter_length, num_events, iordt);
+                }
+                if (cpuid_0xf_1.edx & PQOS_CPUID_MON_TMEM_BW_BIT) {
+
+                        int iordt =
+                            cpuid_0xf_1.eax & PQOS_CPUID_MON_IO_MEM_BW ? 1 : 0;
+
                         add_monitoring_event(mon, 1, PQOS_MON_EVENT_TMEM_BW,
                                              max_rmid, scale_factor,
-                                             counter_length, num_events);
-                if (cpuid_0xf_1.edx & 4)
+                                             counter_length, num_events, iordt);
+                }
+                if (cpuid_0xf_1.edx & PQOS_CPUID_MON_LMEM_BW_BIT) {
+                        int iordt =
+                            cpuid_0xf_1.eax & PQOS_CPUID_MON_IO_MEM_BW ? 1 : 0;
+
                         add_monitoring_event(mon, 1, PQOS_MON_EVENT_LMEM_BW,
                                              max_rmid, scale_factor,
-                                             counter_length, num_events);
+                                             counter_length, num_events, iordt);
+                }
+                if ((cpuid_0xf_1.edx & PQOS_CPUID_MON_TMEM_BW_BIT) &&
+                    (cpuid_0xf_1.edx & PQOS_CPUID_MON_LMEM_BW_BIT)) {
+                        int iordt =
+                            cpuid_0xf_1.eax & PQOS_CPUID_MON_IO_MEM_BW ? 1 : 0;
 
-                if ((cpuid_0xf_1.edx & 2) && (cpuid_0xf_1.edx & 4))
                         add_monitoring_event(mon, 1, PQOS_MON_EVENT_RMEM_BW,
                                              max_rmid, scale_factor,
-                                             counter_length, num_events);
+                                             counter_length, num_events, iordt);
+                }
         }
 
         if (((cpuid_0xa.ebx & 3) == 0) && ((cpuid_0xa.edx & 31) > 1))
                 add_monitoring_event(mon, 0, PQOS_PERF_EVENT_IPC, 0, 0, 0,
-                                     num_events);
+                                     num_events, 0);
 
         if (((cpuid_0xa.eax >> 8) & 0xff) > 1) {
                 add_monitoring_event(mon, 0, PQOS_PERF_EVENT_LLC_MISS, 0, 0, 0,
-                                     num_events);
+                                     num_events, 0);
                 add_monitoring_event(mon, 0, PQOS_PERF_EVENT_LLC_REF, 0, 0, 0,
-                                     num_events);
+                                     num_events, 0);
+        }
+
+        if (mon->iordt) {
+                ret = hw_cap_mon_iordt(cpu, &mon->iordt_on);
+                if (ret != PQOS_RETVAL_OK) {
+                        free(mon);
+                        return ret;
+                }
         }
 
         if (uncore_events & PQOS_PERF_EVENT_LLC_MISS_PCIE_READ)
                 add_monitoring_event(mon, 0, PQOS_PERF_EVENT_LLC_MISS_PCIE_READ,
-                                     0, 0, 0, num_events);
+                                     0, 0, 0, num_events, 0);
         if (uncore_events & PQOS_PERF_EVENT_LLC_MISS_PCIE_WRITE)
                 add_monitoring_event(mon, 0,
                                      PQOS_PERF_EVENT_LLC_MISS_PCIE_WRITE, 0, 0,
-                                     0, num_events);
+                                     0, num_events, 0);
         if (uncore_events & PQOS_PERF_EVENT_LLC_REF_PCIE_READ)
                 add_monitoring_event(mon, 0, PQOS_PERF_EVENT_LLC_REF_PCIE_READ,
-                                     0, 0, 0, num_events);
+                                     0, 0, 0, num_events, 0);
         if (uncore_events & PQOS_PERF_EVENT_LLC_REF_PCIE_WRITE)
                 add_monitoring_event(mon, 0, PQOS_PERF_EVENT_LLC_REF_PCIE_WRITE,
-                                     0, 0, 0, num_events);
+                                     0, 0, 0, num_events, 0);
 
         (*r_mon) = mon;
         return PQOS_RETVAL_OK;
@@ -289,60 +497,16 @@ hw_cap_mon_discover(struct pqos_cap_mon **r_mon, const struct pqos_cpuinfo *cpu)
 int
 hw_cap_l3ca_cdp(const struct pqos_cpuinfo *cpu, int *enabled)
 {
-        unsigned *l3cat_ids = NULL;
-        unsigned l3cat_id_num = 0, j = 0;
-        unsigned enabled_num = 0, disabled_num = 0;
-        int ret = PQOS_RETVAL_OK;
+        return hw_cap_l3_feature(cpu, "L3 CDP", PQOS_MSR_L3_QOS_CFG,
+                                 PQOS_MSR_L3_QOS_CFG_CDP_EN, enabled);
+}
 
-        if (enabled == NULL || cpu == NULL)
-                return PQOS_RETVAL_PARAM;
-
-        /**
-         * Get list of l3cat id's
-         */
-        l3cat_ids = pqos_cpu_get_l3cat_ids(cpu, &l3cat_id_num);
-        if (l3cat_ids == NULL)
-                return PQOS_RETVAL_RESOURCE;
-
-        for (j = 0; j < l3cat_id_num; j++) {
-                uint64_t reg = 0;
-                unsigned core = 0;
-
-                ret = pqos_cpu_get_one_by_l3cat_id(cpu, l3cat_ids[j], &core);
-                if (ret != PQOS_RETVAL_OK)
-                        goto l3cdp_is_enabled_exit;
-
-                if (msr_read(core, PQOS_MSR_L3_QOS_CFG, &reg) !=
-                    MACHINE_RETVAL_OK) {
-                        ret = PQOS_RETVAL_ERROR;
-                        goto l3cdp_is_enabled_exit;
-                }
-
-                if (reg & PQOS_MSR_L3_QOS_CFG_CDP_EN)
-                        enabled_num++;
-                else
-                        disabled_num++;
-        }
-
-        if (disabled_num > 0 && enabled_num > 0) {
-                LOG_ERROR("Inconsistent L3 CDP settings across l3cat_ids."
-                          "Please reset CAT or reboot your system!\n");
-                ret = PQOS_RETVAL_ERROR;
-                goto l3cdp_is_enabled_exit;
-        }
-
-        if (enabled_num > 0)
-                *enabled = 1;
-        else
-                *enabled = 0;
-
-        LOG_INFO("L3 CDP is %s\n", (*enabled) ? "enabled" : "disabled");
-
-l3cdp_is_enabled_exit:
-        if (l3cat_ids != NULL)
-                free(l3cat_ids);
-
-        return ret;
+int
+hw_cap_l3ca_iordt(const struct pqos_cpuinfo *cpu, int *enabled)
+{
+        return hw_cap_l3_feature(cpu, "L3 I/O RDT allocation",
+                                 PQOS_MSR_L3_IO_QOS_CFG,
+                                 PQOS_MSR_L3_IO_QOS_CA_EN, enabled);
 }
 
 /**
@@ -418,12 +582,8 @@ hw_cap_l3ca_brandstr(struct pqos_cap_l3ca *cap)
 #define CPUID_LEAF_BRAND_NUM (CPUID_LEAF_BRAND_END - CPUID_LEAF_BRAND_START + 1)
 #define MAX_BRAND_STRING_LEN (CPUID_LEAF_BRAND_NUM * 4 * sizeof(uint32_t))
         static const char *const supported_brands[] = {
-            "E5-2658 v3", "E5-2648L v3", "E5-2628L v3", "E5-2618L v3",
-            "E5-2608L v3", "E5-2658A v3", "E3-1258L v4", "E3-1278L v4",
-            /* Tiger Lake and Elkhart Lake brand strings */
-            "i3-1115GRE", "i5-1145GRE", "i7-1185GRE", "x6212RE", "x6414RE",
-            "x6425RE", "x6427FE", "x6200FE", "W-11865MRE", "W-11865MLE",
-            "W-11555MRE", "W-11555MLE", "W-11155MRE", "W-11155MLE"};
+            "E5-2658 v3",  "E5-2648L v3", "E5-2628L v3", "E5-2618L v3",
+            "E5-2608L v3", "E5-2658A v3", "E3-1258L v4", "E3-1278L v4"};
         struct cpuid_out res;
         int ret = PQOS_RETVAL_OK;
         int match_found = 0;
@@ -473,13 +633,6 @@ hw_cap_l3ca_brandstr(struct pqos_cap_l3ca *cap)
                 LOG_WARN("Cache allocation not supported on model name '%s'!\n",
                          brand_str);
                 return PQOS_RETVAL_RESOURCE;
-        } else {
-                LOG_WARN(
-                    "Detected model specific non-architectural features (L3 "
-                    "CAT).\n      Intel recommends validating that the feature "
-                    "provides the\n      performance necessary for your "
-                    "use-case. Non-architectural\n      features may not "
-                    "behave as expected in all scenarios.\n");
         }
 
         /**
@@ -558,6 +711,8 @@ hw_cap_l3ca_cpuid(struct pqos_cap_l3ca *cap, const struct pqos_cpuinfo *cpu)
         cap->way_contention = (uint64_t)res.ebx;
         cap->non_contiguous_cbm =
             (res.ecx >> PQOS_CPUID_CAT_NON_CONTIGUOUS_CBM_SUPPORT_BIT) & 1;
+        cap->iordt = (res.ecx >> PQOS_CPUID_CAT_IORDT_BIT) & 1;
+        cap->iordt_on = 0;
 
         if (cap->cdp) {
                 /**
@@ -575,6 +730,15 @@ hw_cap_l3ca_cpuid(struct pqos_cap_l3ca *cap, const struct pqos_cpuinfo *cpu)
                         cap->num_classes = cap->num_classes / 2;
         }
 
+        /* Detect if I/O RDT is enabled */
+        if (cap->iordt) {
+                ret = hw_cap_l3ca_iordt(cpu, &cap->iordt_on);
+                if (ret != PQOS_RETVAL_OK) {
+                        LOG_ERROR("L3 I/O RDT detection error!\n");
+                        return ret;
+                }
+        }
+
         return ret;
 }
 
@@ -647,9 +811,20 @@ hw_cap_l3ca_discover(struct pqos_cap_l3ca *cap, const struct pqos_cpuinfo *cpu)
                         LOG_INFO("Probing msr....\n");
                         ret = hw_cap_l3ca_probe(cap, cpu);
                 }
-                if (ret == PQOS_RETVAL_OK)
+                if (ret == PQOS_RETVAL_OK) {
                         ret =
                             get_cache_info(&cpu->l3, &cap->num_ways, &l3_size);
+
+                        LOG_WARN("Detected model specific non-architectural "
+                                 "features (L3 "
+                                 "CAT).\n      Intel recommends validating "
+                                 "that the feature "
+                                 "provides the\n      performance necessary "
+                                 "for your "
+                                 "use-case. Non-architectural\n      features "
+                                 "may not "
+                                 "behave as expected in all scenarios.\n");
+                }
         }
 
         if (cap->num_ways > 0)
@@ -788,6 +963,73 @@ hw_cap_l2ca_discover(struct pqos_cap_l2ca *cap, const struct pqos_cpuinfo *cpu)
         return ret;
 }
 
+static int
+is_mba40_supported(unsigned core, unsigned *supported)
+{
+        int ret;
+        uint64_t reg = 0;
+
+        *supported = 0;
+
+        ret = msr_read(core, PQOS_MSR_CORE_CAPABILITIES, &reg);
+        if (ret != MACHINE_RETVAL_OK)
+                return PQOS_RETVAL_ERROR;
+
+        if (reg & PQOS_MSR_CORE_CAPABILITIES_MBA40_EN)
+                *supported = 1;
+
+        return PQOS_RETVAL_OK;
+}
+
+static int
+detect_mba40(const struct pqos_cpuinfo *cpu, unsigned *supported)
+{
+        int ret = PQOS_RETVAL_OK;
+        unsigned *mba_ids;
+        unsigned mba_id_num;
+        unsigned i;
+
+        *supported = 0;
+
+        /* MBA IDs */
+        mba_ids = pqos_cpu_get_mba_ids(cpu, &mba_id_num);
+        if (mba_ids == NULL)
+                return PQOS_RETVAL_RESOURCE;
+
+        /*
+         * Detect MBA 4.0 extensions support
+         * Checking all cores to log a debug message
+         */
+        for (i = 0; i < mba_id_num; i++) {
+                unsigned core_supported = 0;
+                unsigned num_cores = 0;
+                unsigned *cores = NULL;
+                unsigned j;
+
+                cores = pqos_cpu_get_cores(cpu, mba_ids[i], &num_cores);
+                if (cores == NULL) {
+                        *supported = 0;
+                        ret = PQOS_RETVAL_ERROR;
+                        break;
+                }
+
+                for (j = 0; j < num_cores; j++) {
+                        ret = is_mba40_supported(cores[j], &core_supported);
+                        if (ret == PQOS_RETVAL_OK && core_supported) {
+                                *supported = 1;
+                                LOG_DEBUG("MBA 4.0 available for core %u\n",
+                                          cores[j]);
+                        }
+                }
+
+                free(cores);
+        }
+
+        free(mba_ids);
+
+        return ret;
+}
+
 int
 hw_cap_mba_discover(struct pqos_cap_mba *cap, const struct pqos_cpuinfo *cpu)
 {
@@ -795,6 +1037,8 @@ hw_cap_mba_discover(struct pqos_cap_mba *cap, const struct pqos_cpuinfo *cpu)
         int ret = PQOS_RETVAL_OK;
         unsigned version;
         unsigned mba_thread_ctrl = 0;
+        unsigned msr_core_caps_available = 0;
+        unsigned mba40_supported = 0;
 
         ASSERT(cpu != NULL);
         ASSERT(cap != NULL);
@@ -814,6 +1058,13 @@ hw_cap_mba_discover(struct pqos_cap_mba *cap, const struct pqos_cpuinfo *cpu)
                 return PQOS_RETVAL_RESOURCE;
         }
 
+        if (res.edx & (1 << 30)) {
+                msr_core_caps_available = 1;
+                LOG_INFO("CPUID.0x7.0: CORE_CAPABILITIES MSR (0x%X) "
+                         "available\n",
+                         PQOS_MSR_CORE_CAPABILITIES);
+        }
+
         /**
          * We can go to CPUID.0x10.0 to obtain more info
          */
@@ -837,10 +1088,18 @@ hw_cap_mba_discover(struct pqos_cap_mba *cap, const struct pqos_cpuinfo *cpu)
 
         /*
          * Detect MBA version
+         *  - MBA4.0 extensions
          *  - MBA3.0 introduces per-thread MBA controls
          *  - MBA2.0 increases number of MBA COS to 15
          */
-        if (res.ecx & 0x1) {
+        if (msr_core_caps_available)
+                detect_mba40(cpu, &mba40_supported);
+
+        if (mba40_supported) {
+                version = 4;
+                cap->mba40 = 1;
+                mba_thread_ctrl = 1;
+        } else if (res.ecx & 0x1) {
                 version = 3;
                 mba_thread_ctrl = 1;
         } else if (cap->num_classes > 8)
@@ -852,7 +1111,7 @@ hw_cap_mba_discover(struct pqos_cap_mba *cap, const struct pqos_cpuinfo *cpu)
         LOG_INFO("Detected Per-%s MBA controls\n",
                  mba_thread_ctrl ? "Thread" : "Core");
 
-        if (version == 2) {
+        if ((version == 2) || (version >= 4)) {
                 unsigned *mba_ids;
                 unsigned mba_id_num;
                 unsigned i;
@@ -878,10 +1137,23 @@ hw_cap_mba_discover(struct pqos_cap_mba *cap, const struct pqos_cpuinfo *cpu)
                                 break;
                         }
 
-                        if (reg & 0x2)
+                        if ((version == 2) && (reg & 0x2))
                                 LOG_INFO(
                                     "MBA Legacy Mode enabled on socket %u\n",
                                     mba_ids[i]);
+
+                        if (version >= 4) {
+                                if (reg & PQOS_MSR_MBA_CFG_MBA40_EN)
+                                        cap->mba40_on = 1;
+                                else
+                                        cap->mba40_on = 0;
+
+                                LOG_INFO(
+                                    "MBA 4.0 extensions %s for socket %u\n",
+                                    cap->mba40_on ? "enabled" : "disabled",
+                                    mba_ids[i]);
+                        }
+
                         if (!mba_thread_ctrl)
                                 LOG_INFO("%s MBA delay enabled on socket %u\n",
                                          (reg & 0x1) ? "Min" : "Max",
