@@ -42,21 +42,20 @@
 
 #include "cap.h"
 #include "cpu_registers.h"
+#include "cpuinfo.h"
+#include "iordt.h"
 #include "log.h"
 #include "machine.h"
 #include "monitoring.h"
 #include "perf_monitoring.h"
 #include "uncore_monitoring.h"
+#include "utils.h"
 
+#include <dirent.h>
+#include <limits.h>
 #include <stdlib.h>
 #include <string.h>
 
-/**
- * ---------------------------------------
- * Local macros
- * ---------------------------------------
- */
-
 /**
  * Special RMID - after reset all cores are associated with it.
  *
@@ -204,6 +203,7 @@ rmid_get_event_max(const struct pqos_cap *cap,
         ret = pqos_cap_get_type(cap, PQOS_CAP_TYPE_MON, &item);
         if (ret != PQOS_RETVAL_OK)
                 return ret;
+
         ASSERT(item != NULL);
         mon = item->u.mon;
 
@@ -229,6 +229,33 @@ rmid_get_event_max(const struct pqos_cap *cap,
         return PQOS_RETVAL_OK;
 }
 
+/**
+ * @brief Obtain socket number for \a numa
+ *
+ * @param [in] cpu CPU information structure
+ * @param [in] numa NUMA node
+ * @param [out] socket obtained socket id
+ *
+ * @return Operational status
+ * @retval PQOS_RETVAL_OK success
+ */
+static int
+get_socket(const struct pqos_cpuinfo *cpu, unsigned numa, unsigned *socket)
+{
+        unsigned i;
+
+        for (i = 0; i < cpu->num_cores; ++i) {
+                const struct pqos_coreinfo *coreinfo = &(cpu->cores[i]);
+
+                if (coreinfo->numa == numa) {
+                        *socket = coreinfo->socket;
+                        return PQOS_RETVAL_OK;
+                }
+        }
+
+        return PQOS_RETVAL_ERROR;
+}
+
 int
 hw_mon_assoc_unused(struct pqos_mon_poll_ctx *ctx,
                     const enum pqos_mon_event event,
@@ -238,18 +265,19 @@ hw_mon_assoc_unused(struct pqos_mon_poll_ctx *ctx,
 {
         const struct pqos_cpuinfo *cpu = _pqos_get_cpu();
         const struct pqos_cap *cap = _pqos_get_cap();
+        const struct pqos_devinfo *dev = _pqos_get_dev();
         int ret = PQOS_RETVAL_OK;
-        pqos_rmid_t rmid;
+        unsigned rmid = 0;
         unsigned *core_list = NULL;
         unsigned i, core_count;
         uint8_t *rmid_list = NULL;
+        int iordt;
 
         ASSERT(ctx != NULL);
 
 #ifndef PQOS_RMID_CUSTOM
         UNUSED_PARAM(opt);
 #endif
-
         /* Getting max RMID for given event */
         ret = rmid_get_event_max(cap, &rmid, event);
         if (ret != PQOS_RETVAL_OK)
@@ -259,8 +287,12 @@ hw_mon_assoc_unused(struct pqos_mon_poll_ctx *ctx,
         if (min_rmid < 1)
                 min_rmid = 1;
 
+        ret = pqos_mon_iordt_enabled(cap, NULL, &iordt);
+        if (ret != PQOS_RETVAL_OK)
+                goto rmid_alloc_error;
+
         /* list of used RMIDs */
-        rmid_list = (uint8_t *)calloc(max_rmid + 2, sizeof(*rmid_list));
+        rmid_list = (uint8_t *)calloc(max_rmid + 2, sizeof(rmid_list[0]));
         if (rmid_list == NULL)
                 return PQOS_RETVAL_RESOURCE;
 
@@ -273,6 +305,7 @@ hw_mon_assoc_unused(struct pqos_mon_poll_ctx *ctx,
                 goto rmid_alloc_error;
         }
         ASSERT(core_count > 0);
+
         /* Mark RMIDs used for core monitoring */
         for (i = 0; i < core_count; i++) {
                 pqos_rmid_t rmid;
@@ -284,6 +317,34 @@ hw_mon_assoc_unused(struct pqos_mon_poll_ctx *ctx,
                         rmid_list[rmid] = 1;
         }
 
+        /* mark used RMIDs for channels */
+        if (iordt && dev != NULL)
+                for (i = 0; i < dev->num_channels; ++i) {
+                        const struct pqos_channel *channel = &dev->channels[i];
+                        pqos_rmid_t rmid;
+                        unsigned socket;
+                        unsigned numa;
+
+                        if (!channel->rmid_tagging)
+                                continue;
+
+                        ret = iordt_get_numa(dev, channel->channel_id, &numa);
+                        if (ret == PQOS_RETVAL_OK)
+                                ret = get_socket(cpu, numa, &socket);
+                        if (ret == PQOS_RETVAL_OK) {
+                                if (socket != ctx->cluster)
+                                        continue;
+                        } else if (ret != PQOS_RETVAL_RESOURCE)
+                                goto rmid_alloc_error;
+
+                        ret = iordt_mon_assoc_read(channel->channel_id, &rmid);
+                        if (ret != PQOS_RETVAL_OK)
+                                goto rmid_alloc_error;
+
+                        if (rmid <= max_rmid)
+                                rmid_list[rmid] = 1;
+                }
+
 #ifdef PQOS_RMID_CUSTOM
         if (opt->rmid.type == PQOS_RMID_TYPE_MAP) {
                 if (opt->rmid.rmid < min_rmid || opt->rmid.rmid > max_rmid) {
@@ -351,17 +412,20 @@ static uint64_t
 scale_event(const enum pqos_mon_event event, const uint64_t val)
 {
         const struct pqos_cap *cap = _pqos_get_cap();
+        const struct pqos_capability *cap_mon =
+            _pqos_cap_get_type(PQOS_CAP_TYPE_MON);
         const struct pqos_monitor *pmon;
         int ret;
 
         ASSERT(cap != NULL);
+        ASSERT(cap_mon != NULL);
 
         ret = pqos_cap_get_event(cap, event, &pmon);
         ASSERT(ret == PQOS_RETVAL_OK);
         if (ret != PQOS_RETVAL_OK)
                 return val;
         else
-                return val * pmon->scale_factor;
+                return val * pmon->scale_factor / cap_mon->u.mon->snc_num;
 }
 
 int
@@ -406,7 +470,7 @@ hw_mon_assoc_read(const unsigned lcore, pqos_rmid_t *rmid)
 }
 
 int
-hw_mon_assoc_get(const unsigned lcore, pqos_rmid_t *rmid)
+hw_mon_assoc_get_core(const unsigned lcore, pqos_rmid_t *rmid)
 {
         int ret = PQOS_RETVAL_OK;
         const struct pqos_cpuinfo *cpu = _pqos_get_cpu();
@@ -423,13 +487,152 @@ hw_mon_assoc_get(const unsigned lcore, pqos_rmid_t *rmid)
         return ret;
 }
 
+static int
+hw_mon_set_snc_mode(const struct pqos_cpuinfo *cpu, enum pqos_snc_config ns)
+{
+        uint64_t val = 0;
+        uint32_t reg = PQOS_MSR_SNC_CFG;
+        unsigned i, sock_count, *sockets = NULL;
+        int ret;
+
+        if (ns != PQOS_REQUIRE_SNC_TOTAL && ns != PQOS_REQUIRE_SNC_LOCAL)
+                return PQOS_RETVAL_PARAM;
+
+        if (ns == PQOS_REQUIRE_SNC_LOCAL)
+                LOG_INFO("Turning SNC to local mode ...\n");
+        else if (ns == PQOS_REQUIRE_SNC_TOTAL)
+                LOG_INFO("Turning SNC to total mode ...\n");
+
+        sockets = pqos_cpu_get_sockets(cpu, &sock_count);
+        if (sockets == NULL || sock_count == 0) {
+                printf("Error retrieving information for Sockets\n");
+                goto return_error;
+        }
+
+        for (i = 0; i < sock_count; i++) {
+                unsigned lcore;
+
+                ret = pqos_cpu_get_one_core(cpu, sockets[i], &lcore);
+                if (ret != PQOS_RETVAL_OK) {
+                        printf("Error retrieving lcore for socket %u\n",
+                               sockets[i]);
+                        goto return_error;
+                };
+
+                if (msr_read(lcore, reg, &val) != MACHINE_RETVAL_OK)
+                        goto return_error;
+
+                val &= ~1;
+                if (ns == PQOS_REQUIRE_SNC_LOCAL)
+                        val |= 1;
+
+                if (msr_write(lcore, reg, val) != MACHINE_RETVAL_OK)
+                        goto return_error;
+        };
+
+        _pqos_cap_mon_snc_change(ns);
+
+        ret = PQOS_RETVAL_OK;
+        goto clean_up;
+return_error:
+        ret = PQOS_RETVAL_ERROR;
+clean_up:
+        if (sockets)
+                free(sockets);
+
+        return ret;
+}
+
 int
-hw_mon_reset(void)
+hw_mon_assoc_get_channel(const pqos_channel_t channel_id, pqos_rmid_t *rmid)
+{
+        const struct pqos_cap *cap = _pqos_get_cap();
+        const struct pqos_devinfo *dev = _pqos_get_dev();
+        const struct pqos_channel *channel;
+        int ret;
+        int supported;
+        int enabled;
+
+        if (rmid == NULL)
+                return PQOS_RETVAL_PARAM;
+
+        channel = pqos_devinfo_get_channel(dev, channel_id);
+        if (channel == NULL)
+                return PQOS_RETVAL_PARAM;
+        if (!channel->rmid_tagging)
+                return PQOS_RETVAL_PARAM;
+
+        ret = pqos_mon_iordt_enabled(cap, &supported, &enabled);
+        if (ret != PQOS_RETVAL_OK)
+                return ret;
+        if (!supported)
+                return PQOS_RETVAL_RESOURCE;
+        if (!enabled)
+                return PQOS_RETVAL_ERROR;
+
+        return iordt_mon_assoc_read(channel_id, rmid);
+}
+
+int
+hw_mon_reset_iordt(const struct pqos_cpuinfo *cpu, const int enable)
+{
+        unsigned *sockets = NULL;
+        unsigned sockets_num;
+        unsigned j = 0;
+        int ret = PQOS_RETVAL_OK;
+
+        ASSERT(cpu != NULL);
+
+        LOG_INFO("%s I/O RDT monitoring across sockets...\n",
+                 (enable) ? "Enabling" : "Disabling");
+
+        sockets = pqos_cpu_get_sockets(cpu, &sockets_num);
+        if (sockets == NULL || sockets_num == 0) {
+                ret = PQOS_RETVAL_ERROR;
+                goto hw_mon_reset_iordt_exit;
+        }
+
+        for (j = 0; j < sockets_num; j++) {
+                uint64_t reg = 0;
+                unsigned core = 0;
+
+                ret = pqos_cpu_get_one_core(cpu, sockets[j], &core);
+                if (ret != PQOS_RETVAL_OK)
+                        goto hw_mon_reset_iordt_exit;
+
+                ret = msr_read(core, PQOS_MSR_L3_IO_QOS_CFG, &reg);
+                if (ret != MACHINE_RETVAL_OK) {
+                        ret = PQOS_RETVAL_ERROR;
+                        goto hw_mon_reset_iordt_exit;
+                }
+
+                if (enable)
+                        reg |= PQOS_MSR_L3_IO_QOS_MON_EN;
+                else
+                        reg &= ~PQOS_MSR_L3_IO_QOS_MON_EN;
+
+                ret = msr_write(core, PQOS_MSR_L3_IO_QOS_CFG, reg);
+                if (ret != MACHINE_RETVAL_OK) {
+                        ret = PQOS_RETVAL_ERROR;
+                        goto hw_mon_reset_iordt_exit;
+                }
+        }
+
+hw_mon_reset_iordt_exit:
+        if (sockets != NULL)
+                free(sockets);
+
+        return ret;
+}
+
+int
+hw_mon_reset(const struct pqos_mon_config *cfg)
 {
         int ret = PQOS_RETVAL_OK;
         unsigned i;
         const struct pqos_cpuinfo *cpu = _pqos_get_cpu();
         const struct pqos_cap *cap = _pqos_get_cap();
+        const struct pqos_devinfo *dev = _pqos_get_dev();
         const struct pqos_capability *cap_mon;
 
         ret = pqos_cap_get_type(cap, PQOS_CAP_TYPE_MON, &cap_mon);
@@ -438,6 +641,14 @@ hw_mon_reset(void)
                 return ret;
         }
 
+        if (cfg != NULL && cfg->l3_iordt == PQOS_REQUIRE_IORDT_ON &&
+            !cap_mon->u.mon->iordt) {
+                LOG_ERROR("I/O RDT monitoring requested but not supported by "
+                          "the platform!\n");
+                return PQOS_RETVAL_PARAM;
+        }
+
+        /* reset core assoc */
         for (i = 0; i < cpu->num_cores; i++) {
                 int retval = hw_mon_assoc_write(cpu->cores[i].lcore, RMID0);
 
@@ -445,6 +656,54 @@ hw_mon_reset(void)
                         ret = retval;
         }
 
+        /* reset I/O RDT channel assoc */
+        if (cap_mon->u.mon->iordt && cap_mon->u.mon->iordt_on && dev != NULL) {
+                int retval = iordt_mon_assoc_reset(dev);
+
+                if (retval != PQOS_RETVAL_OK)
+                        ret = retval;
+        }
+
+        if (ret != PQOS_RETVAL_OK)
+                return ret;
+
+        if (cfg != NULL) {
+                if (cfg->l3_iordt == PQOS_REQUIRE_IORDT_ON &&
+                    !cap_mon->u.mon->iordt_on) {
+                        LOG_INFO("Turning I/O RDT Monitoring ON ...\n");
+                        ret = hw_mon_reset_iordt(cpu, 1);
+                        if (ret != PQOS_RETVAL_OK) {
+                                LOG_ERROR("I/O RDT Monitoring enable error!\n");
+                                return ret;
+                        }
+
+                        /* reset channel assoc - initialize mmio tables */
+                        if (dev != NULL)
+                                ret = iordt_mon_assoc_reset(dev);
+                }
+
+                if (cfg->l3_iordt == PQOS_REQUIRE_IORDT_OFF &&
+                    cap_mon->u.mon->iordt_on) {
+                        LOG_INFO("Turning I/O RDT Monitoring OFF ...\n");
+                        ret = hw_mon_reset_iordt(cpu, 0);
+                        if (ret != PQOS_RETVAL_OK) {
+                                LOG_ERROR("I/O RDT Monitoring disable "
+                                          "error!\n");
+                                return ret;
+                        }
+                }
+                _pqos_cap_mon_iordt_change(cfg->l3_iordt);
+
+                if (cfg->snc != PQOS_REQUIRE_SNC_ANY) {
+                        if (cap_mon->u.mon->snc_num == 1) {
+                                LOG_ERROR("SNC requested but not supported by "
+                                          "the platform!\n");
+                                ret = PQOS_RETVAL_PARAM;
+                        } else
+                                ret = hw_mon_set_snc_mode(cpu, cfg->snc);
+                }
+        }
+
         return ret;
 }
 
@@ -786,16 +1045,24 @@ hw_mon_start_counter(struct pqos_mon_data *group,
 {
         const unsigned num_cores = group->num_cores;
         const struct pqos_cpuinfo *cpu = _pqos_get_cpu();
-        unsigned core2cluster[num_cores];
-        struct pqos_mon_poll_ctx ctxs[num_cores];
+        const struct pqos_capability *cap_mon =
+            _pqos_cap_get_type(PQOS_CAP_TYPE_MON);
+        pqos_rmid_t core2rmid[num_cores];
+        struct pqos_mon_poll_ctx *ctxs = NULL;
         unsigned num_ctxs = 0;
         unsigned i;
         int ret = PQOS_RETVAL_OK;
         enum pqos_mon_event ctx_event = (enum pqos_mon_event)(
             event & (PQOS_MON_EVENT_L3_OCCUP | PQOS_MON_EVENT_LMEM_BW |
                      PQOS_MON_EVENT_TMEM_BW | PQOS_MON_EVENT_RMEM_BW));
+        pqos_rmid_t rmid_min = 1;
+        pqos_rmid_t rmid_numa =
+            cap_mon->u.mon->max_rmid / cap_mon->u.mon->snc_num;
+        pqos_rmid_t rmid_max = rmid_numa - 1;
 
-        memset(ctxs, 0, sizeof(ctxs));
+        ctxs = calloc(num_cores * cap_mon->u.mon->snc_num, sizeof(*ctxs));
+        if (ctxs == NULL)
+                return PQOS_RETVAL_RESOURCE;
 
         /*
          * Initialize poll context table:
@@ -806,18 +1073,41 @@ hw_mon_start_counter(struct pqos_mon_data *group,
                 const unsigned lcore = group->cores[i];
                 unsigned j;
                 unsigned cluster = 0;
+                unsigned numa = 0;
 
                 ret = pqos_cpu_get_clusterid(cpu, lcore, &cluster);
-                if (ret != PQOS_RETVAL_OK)
-                        return PQOS_RETVAL_PARAM;
-                core2cluster[i] = cluster;
+                if (ret != PQOS_RETVAL_OK) {
+                        ret = PQOS_RETVAL_PARAM;
+                        goto hw_mon_start_counter_exit;
+                }
 
+                /* When SNC in default mode RMID's are assigned on numa node
+                 * basis
+                 */
+                if (cap_mon->u.mon->snc_num > 1 &&
+                    cap_mon->u.mon->snc_mode == PQOS_SNC_LOCAL) {
+                        ret = pqos_cpu_get_numaid(cpu, lcore, &numa);
+                        if (ret != PQOS_RETVAL_OK) {
+                                ret = PQOS_RETVAL_PARAM;
+                                goto hw_mon_start_counter_exit;
+                        }
+                        numa = numa % cap_mon->u.mon->snc_num;
+                        rmid_min = rmid_numa * numa + 1;
+                        rmid_max = rmid_numa * (numa + 1) - 1;
+                }
+
+                /* CORES in the same coluster/numa node share RMID */
                 for (j = 0; j < num_ctxs; j++)
                         if (ctxs[j].lcore == lcore ||
-                            ctxs[j].cluster == cluster)
+                            (ctxs[j].cluster == cluster &&
+                             ctxs[j].numa == numa)) {
+                                core2rmid[i] = ctxs[j].rmid;
                                 break;
+                        }
 
                 if (j >= num_ctxs) {
+                        struct pqos_mon_poll_ctx *ctx = &ctxs[num_ctxs];
+
                         /**
                          * New cluster is found
                          * - save cluster id in the table
@@ -825,20 +1115,36 @@ hw_mon_start_counter(struct pqos_mon_data *group,
                          */
                         ctxs[num_ctxs].lcore = lcore;
                         ctxs[num_ctxs].cluster = cluster;
+                        ctxs[num_ctxs].numa = numa;
 
-                        ret = hw_mon_assoc_unused(&ctxs[num_ctxs], ctx_event, 1,
-                                                  UINT32_MAX, opt);
+                        ret = hw_mon_assoc_unused(&ctxs[num_ctxs], ctx_event,
+                                                  rmid_min, rmid_max, opt);
                         if (ret != PQOS_RETVAL_OK)
-                                return ret;
+                                goto hw_mon_start_counter_exit;
 
+                        core2rmid[i] = ctx->rmid;
                         num_ctxs++;
+
+                        /* In shared mode - monitor all numa nodes */
+                        if (cap_mon->u.mon->snc_mode == PQOS_SNC_TOTAL) {
+                                unsigned c;
+
+                                for (c = 1; c < cap_mon->u.mon->snc_num; ++c) {
+                                        ctxs[num_ctxs] = ctxs[num_ctxs - 1];
+                                        ctxs[num_ctxs].rmid += rmid_numa;
+                                        ctxs[num_ctxs].quiet = 1;
+                                        num_ctxs++;
+                                }
+                        }
                 }
         }
 
-        group->intl->hw.ctx = (struct pqos_mon_poll_ctx *)calloc(
-            num_ctxs, sizeof(group->intl->hw.ctx[0]));
-        if (group->intl->hw.ctx == NULL)
-                return PQOS_RETVAL_RESOURCE;
+        group->intl->hw.ctx = realloc(ctxs, num_ctxs * sizeof(*ctxs));
+        if (group->intl->hw.ctx == NULL) {
+                ret = PQOS_RETVAL_RESOURCE;
+                goto hw_mon_start_counter_exit;
+        }
+        ctxs = NULL;
 
         /**
          * Associate requested cores with
@@ -846,68 +1152,63 @@ hw_mon_start_counter(struct pqos_mon_data *group,
          */
         group->num_cores = num_cores;
         for (i = 0; i < num_cores; i++) {
-                unsigned cluster, j;
-                pqos_rmid_t rmid;
-
-                cluster = core2cluster[i];
-                for (j = 0; j < num_ctxs; j++)
-                        if (ctxs[j].cluster == cluster)
-                                break;
-                if (j >= num_ctxs) {
-                        ret = PQOS_RETVAL_ERROR;
-                        goto hw_mon_start_counter_exit;
-                }
-                rmid = ctxs[j].rmid;
+                pqos_rmid_t rmid = core2rmid[i];
 
                 ret = hw_mon_assoc_write(group->cores[i], rmid);
                 if (ret != PQOS_RETVAL_OK)
-                        goto hw_mon_start_counter_exit;
+                        break;
         }
 
-        group->intl->hw.num_ctx = num_ctxs;
-        for (i = 0; i < num_ctxs; i++)
-                group->intl->hw.ctx[i] = ctxs[i];
-
-        group->intl->hw.event |= ctx_event;
-
-hw_mon_start_counter_exit:
-        if (ret != PQOS_RETVAL_OK) {
+        if (ret == PQOS_RETVAL_OK) {
+                group->intl->hw.num_ctx = num_ctxs;
+                group->intl->hw.event |= ctx_event;
+        } else {
                 for (i = 0; i < num_cores; i++)
                         (void)hw_mon_assoc_write(group->cores[i], RMID0);
-
                 if (group->intl->hw.ctx != NULL)
                         free(group->intl->hw.ctx);
         }
 
+hw_mon_start_counter_exit:
+        if (ctxs != NULL)
+                free(ctxs);
+
         return ret;
 }
 
 /**
- * @brief Validate if event is listed in capabilities
+ * @brief Validate if event list contains events listed in capabilities
  *
- * @param [in[ cap capabilities structure
- * @param [in] event monitoring event id
+ * @param [in] event combination of monitoring events
+ * @param [in] iordt require I/O RDT support
  *
- * @return Operational status
- * @retval PQOS_RETVAL_OK success
+ * @return Operations status
+ * @retval PQOS_RETVAL_OK on success
  */
 static int
-validate_event(const struct pqos_cap *cap, const enum pqos_mon_event event)
+hw_mon_events_valid(const struct pqos_cap *cap,
+                    const enum pqos_mon_event event,
+                    const int iordt)
 {
         unsigned i;
 
-        for (i = 0; i < (sizeof(event) * 8); i++) {
+        /**
+         * Validate if event is listed in capabilities
+         */
+        for (i = 0; i < (sizeof(event) * CHAR_BIT); i++) {
+                int retval;
                 const enum pqos_mon_event evt_mask =
                     (enum pqos_mon_event)(1U << i);
                 const struct pqos_monitor *ptr = NULL;
-                int retval;
 
                 if (!(evt_mask & event))
                         continue;
 
                 retval = pqos_cap_get_event(cap, evt_mask, &ptr);
                 if (retval != PQOS_RETVAL_OK || ptr == NULL)
-                        return PQOS_RETVAL_PARAM;
+                        return PQOS_RETVAL_ERROR;
+                if (iordt && !ptr->iordt)
+                        return PQOS_RETVAL_ERROR;
         }
 
         return PQOS_RETVAL_OK;
@@ -934,6 +1235,9 @@ hw_mon_start_cores(const unsigned num_cores,
         ASSERT(num_cores > 0);
         ASSERT(event > 0);
 
+        if (num_cores == 0)
+                return PQOS_RETVAL_PARAM;
+
         req_events = event;
 
         if (req_events & PQOS_MON_EVENT_RMEM_BW)
@@ -943,9 +1247,12 @@ hw_mon_start_cores(const unsigned num_cores,
                 req_events |= (enum pqos_mon_event)(
                     PQOS_PERF_EVENT_CYCLES | PQOS_PERF_EVENT_INSTRUCTIONS);
 
-        ret = validate_event(cap, event);
+        /**
+         * Validate if event is listed in capabilities
+         */
+        ret = hw_mon_events_valid(cap, event, 0);
         if (ret != PQOS_RETVAL_OK)
-                return ret;
+                return PQOS_RETVAL_PARAM;
 
         /**
          * Check if all requested cores are valid
@@ -1056,7 +1363,7 @@ hw_mon_start_uncore(const unsigned num_sockets,
         ASSERT(num_sockets > 0);
         ASSERT(event > 0);
 
-        ret = validate_event(cap, event);
+        ret = hw_mon_events_valid(cap, event, 0);
         if (ret != PQOS_RETVAL_OK)
                 return ret;
 
@@ -1106,6 +1413,207 @@ hw_mon_start_uncore__exit:
         return ret;
 }
 
+int
+hw_mon_start_channels(const unsigned num_channels,
+                      const pqos_channel_t *channels,
+                      const enum pqos_mon_event event,
+                      void *context,
+                      struct pqos_mon_data *group,
+                      const struct pqos_mon_options *opt)
+{
+        int ret = PQOS_RETVAL_OK;
+        unsigned i;
+        const struct pqos_devinfo *dev = _pqos_get_dev();
+        const struct pqos_cap *cap = _pqos_get_cap();
+        const struct pqos_cpuinfo *cpu = _pqos_get_cpu();
+        struct pqos_mon_poll_ctx *ctxs = NULL;
+        unsigned num_ctx = 0;
+        int enabled;
+        int supported;
+        enum pqos_mon_event req_events;
+
+        ASSERT(group != NULL);
+        ASSERT(channels != NULL);
+        ASSERT(num_channels > 0);
+        ASSERT(event > 0);
+        ASSERT(opt != NULL);
+
+#ifdef PQOS_RMID_CUSTOM
+        if (opt->rmid.type != PQOS_RMID_TYPE_DEFAULT &&
+            opt->rmid.type != PQOS_RMID_TYPE_MAP)
+                return PQOS_RETVAL_PARAM;
+#endif
+
+        req_events = event;
+        if (req_events & PQOS_MON_EVENT_RMEM_BW)
+                req_events |= (enum pqos_mon_event)(PQOS_MON_EVENT_LMEM_BW |
+                                                    PQOS_MON_EVENT_TMEM_BW);
+
+        /* Check for I/O RDT support */
+        ret = pqos_mon_iordt_enabled(cap, &supported, &enabled);
+        if (ret != PQOS_RETVAL_OK)
+                return ret;
+        if (!supported) {
+                LOG_ERROR("I/O RDT monitoring is not supported!\n");
+                return PQOS_RETVAL_RESOURCE;
+        } else if (!enabled) {
+                LOG_ERROR("I/O RDT monitoring is disabled!\n");
+                return PQOS_RETVAL_ERROR;
+        }
+
+        /* Validate if event is listed in capabilities */
+        ret = hw_mon_events_valid(cap, event, 1);
+        if (ret != PQOS_RETVAL_OK)
+                return PQOS_RETVAL_PARAM;
+
+        /* Check if all requested channels are valid */
+        for (i = 0; i < num_channels; ++i) {
+                const pqos_channel_t channel_id = channels[i];
+                const struct pqos_channel *channel;
+                pqos_rmid_t rmid;
+
+                channel = pqos_devinfo_get_channel(dev, channel_id);
+                if (channel == NULL)
+                        return PQOS_RETVAL_PARAM;
+                if (!channel->rmid_tagging) {
+                        LOG_ERROR(
+                            "Channel %016llx does not support monitoring\n",
+                            (unsigned long long)channel_id);
+                        return PQOS_RETVAL_RESOURCE;
+                }
+
+                ret = iordt_mon_assoc_read(channel_id, &rmid);
+                if (ret != PQOS_RETVAL_OK)
+                        return PQOS_RETVAL_PARAM;
+
+                if (rmid != RMID0) {
+                        /* If not RMID0 then it is already monitored */
+                        LOG_INFO("Channel %16llx is already monitored with "
+                                 "RMID%u.\n",
+                                 (unsigned long long)channel, rmid);
+                        return PQOS_RETVAL_RESOURCE;
+                }
+        }
+
+        ctxs = (struct pqos_mon_poll_ctx *)calloc(num_channels, sizeof(*ctxs));
+        if (ctxs == NULL)
+                return PQOS_RETVAL_RESOURCE;
+
+        for (i = 0; i < num_channels; ++i) {
+                const pqos_channel_t channel_id = channels[i];
+                const struct pqos_channel *channel =
+                    pqos_devinfo_get_channel(dev, channel_id);
+                struct pqos_mon_poll_ctx *ctx = NULL;
+                const struct pqos_capability *item = NULL;
+                unsigned numa;
+                unsigned socket;
+                unsigned c;
+
+                if (channel == NULL) {
+                        ret = PQOS_RETVAL_PARAM;
+                        goto hw_mon_start_channels_exit;
+                }
+
+                /* Device assigned to numa */
+                ret = iordt_get_numa(dev, channel_id, &numa);
+                if (ret != PQOS_RETVAL_OK)
+                        goto hw_mon_start_channels_exit;
+
+                /* Obtain socket number */
+                ret = get_socket(cpu, numa, &socket);
+                if (ret != PQOS_RETVAL_OK)
+                        goto hw_mon_start_channels_exit;
+
+                ret = pqos_cap_get_type(cap, PQOS_CAP_TYPE_MON, &item);
+                if (ret != PQOS_RETVAL_OK)
+                        goto hw_mon_start_channels_exit;
+
+                if (item->u.mon->max_rmid == 0) {
+                        ret = PQOS_RETVAL_PARAM;
+                        goto hw_mon_start_channels_exit;
+                }
+
+                /* check if we can reuse context already exists */
+                for (c = 0; c < num_ctx; ++c)
+                        if (ctxs[c].cluster == socket &&
+                            ctxs[c].rmid <= item->u.mon->max_rmid)
+                                ctx = &ctxs[c];
+                if (ctx != NULL) {
+                        ret = iordt_mon_assoc_write(channel_id, ctx->rmid);
+                        if (ret != PQOS_RETVAL_OK)
+                                goto hw_mon_start_channels_exit;
+                        continue;
+                }
+
+                ctx = &ctxs[num_ctx++];
+                ctx->cluster = socket;
+
+                ret = pqos_cpu_get_one_core(cpu, ctx->cluster, &ctx->lcore);
+                if (ret != PQOS_RETVAL_OK)
+                        goto hw_mon_start_channels_exit;
+
+                ret = hw_mon_assoc_unused(ctx, event, 0, item->u.mon->max_rmid,
+                                          opt);
+                if (ret != PQOS_RETVAL_OK)
+                        goto hw_mon_start_channels_exit;
+
+                ret = iordt_mon_assoc_write(channel_id, ctx->rmid);
+                if (ret != PQOS_RETVAL_OK)
+                        goto hw_mon_start_channels_exit;
+        }
+
+        /* shrink memory used by ctx */
+        /* ignore scan-build false positives due to realloc use */
+#ifndef __clang_analyzer__
+        {
+                struct pqos_mon_poll_ctx *ctxs_ptr =
+                    (struct pqos_mon_poll_ctx *)realloc(
+                        ctxs, num_ctx * sizeof(*ctxs));
+
+                if (ctxs_ptr == NULL) {
+                        ret = PQOS_RETVAL_ERROR;
+                        goto hw_mon_start_channels_exit;
+                }
+
+                ctxs = ctxs_ptr;
+        }
+#endif
+
+        /* Fill in the monitoring group structure */
+        group->event = event;
+        group->context = context;
+        group->num_channels = num_channels;
+        group->channels =
+            (pqos_channel_t *)malloc(sizeof(group->channels[0]) * num_channels);
+        if (group->channels == NULL) {
+                ret = PQOS_RETVAL_RESOURCE;
+                goto hw_mon_start_channels_exit;
+        }
+        for (i = 0; i < group->num_channels; i++)
+                group->channels[i] = channels[i];
+
+        group->intl->hw.num_ctx = num_ctx;
+        group->intl->hw.ctx = ctxs;
+        group->intl->hw.event |= req_events;
+
+hw_mon_start_channels_exit:
+        if (ret != PQOS_RETVAL_OK) {
+                if (ctxs != NULL) {
+                        /* Assign channels back to RMID0 */
+                        for (i = 0; i < num_channels; ++i) {
+                                const pqos_channel_t channel_id = channels[i];
+
+                                iordt_mon_assoc_write(channel_id, RMID0);
+                        }
+                        free(ctxs);
+                }
+                if (group->channels != NULL)
+                        free(group->channels);
+        }
+
+        return ret;
+}
+
 int
 hw_mon_stop(struct pqos_mon_data *group)
 {
@@ -1116,11 +1624,18 @@ hw_mon_stop(struct pqos_mon_data *group)
 
         ASSERT(group != NULL);
 
-        if (group->num_cores == 0 && group->intl->uncore.num_sockets == 0) {
+        if ((group->num_cores == 0 && group->num_channels == 0 &&
+             group->intl->uncore.num_sockets == 0))
                 return PQOS_RETVAL_PARAM;
-        } else if (group->num_cores > 0 &&
-                   (group->cores == NULL || group->intl->hw.num_ctx == 0 ||
-                    group->intl->hw.ctx == NULL))
+        if (group->num_cores != 0 && group->cores == NULL)
+                return PQOS_RETVAL_PARAM;
+        if (group->num_channels != 0 && group->channels == NULL)
+                return PQOS_RETVAL_PARAM;
+        if (group->intl->uncore.num_sockets != 0 &&
+            group->intl->uncore.sockets == NULL)
+                return PQOS_RETVAL_PARAM;
+        if ((group->num_cores > 0 || group->num_channels > 0) &&
+            (group->intl->hw.num_ctx == 0 || group->intl->hw.ctx == NULL))
                 return PQOS_RETVAL_PARAM;
 
         for (i = 0; i < group->intl->hw.num_ctx; i++) {
@@ -1130,26 +1645,37 @@ hw_mon_stop(struct pqos_mon_data *group)
                 const unsigned lcore = group->intl->hw.ctx[i].lcore;
                 pqos_rmid_t rmid = RMID0;
 
+                if (!group->num_cores)
+                        break;
+
                 ret = pqos_cpu_check_core(cpu, lcore);
                 if (ret != PQOS_RETVAL_OK)
                         return PQOS_RETVAL_PARAM;
                 ret = hw_mon_assoc_read(lcore, &rmid);
                 if (ret != PQOS_RETVAL_OK)
                         return PQOS_RETVAL_PARAM;
-                if (rmid != group->intl->hw.ctx[i].rmid)
+                if (rmid != group->intl->hw.ctx[i].rmid &&
+                    !group->intl->hw.ctx[i].quiet)
                         LOG_WARN("Core %u RMID association changed from %u "
                                  "to %u! The core has been hijacked!\n",
                                  lcore, group->intl->hw.ctx[i].rmid, rmid);
         }
 
-        for (i = 0; i < group->num_cores; i++) {
-                /**
-                 * Associate cores from the group back with RMID0
-                 */
-                ret = hw_mon_assoc_write(group->cores[i], RMID0);
-                if (ret != PQOS_RETVAL_OK)
-                        retval = PQOS_RETVAL_RESOURCE;
-        }
+        /* Associate cores from the group back with RMID0 */
+        if (group->cores != NULL)
+                for (i = 0; i < group->num_cores; ++i) {
+                        ret = hw_mon_assoc_write(group->cores[i], RMID0);
+                        if (ret != PQOS_RETVAL_OK)
+                                retval = PQOS_RETVAL_RESOURCE;
+                }
+
+        /* Associate channels from the group back with RMID0 */
+        if (group->channels != NULL)
+                for (i = 0; i < group->num_channels; ++i) {
+                        ret = iordt_mon_assoc_write(group->channels[i], RMID0);
+                        if (ret != PQOS_RETVAL_OK)
+                                retval = PQOS_RETVAL_RESOURCE;
+                }
 
         /* stop perf counters */
         ret = hw_mon_stop_perf(group);
@@ -1164,7 +1690,10 @@ hw_mon_stop(struct pqos_mon_data *group)
         /**
          * Free poll contexts, core list and clear the group structure
          */
-        free(group->cores);
+        if (group->cores != NULL)
+                free(group->cores);
+        if (group->channels != NULL)
+                free(group->channels);
         free(group->intl->hw.ctx);
 
         return retval;
@@ -1208,7 +1737,7 @@ hw_mon_read_counter(struct pqos_mon_data *group,
 
         switch (event) {
         case PQOS_MON_EVENT_L3_OCCUP:
-                pv->llc = scale_event(PQOS_MON_EVENT_L3_OCCUP, value);
+                pv->llc = scale_event(event, value);
                 break;
         case PQOS_MON_EVENT_LMEM_BW:
                 if (pmon->counter_length == 32 && pv->mbm_local > value)
@@ -1289,13 +1818,8 @@ hw_mon_read_perf(struct pqos_mon_data *group, const enum pqos_mon_event event)
                 break;
         case PQOS_PERF_EVENT_LLC_REF:
                 reg = IA32_MSR_PMC1;
-#if PQOS_VERSION < 50000
-                value = &group->intl->values.llc_references;
-                delta = &group->intl->values.llc_references_delta;
-#else
                 value = &values->llc_references;
                 delta = &values->llc_references_delta;
-#endif
                 break;
         default:
                 return PQOS_RETVAL_PARAM;
