@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2019-2021 Intel Corporation. All rights reserved.
+ * Copyright(c) 2019-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -30,22 +30,21 @@
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+#include "os_cap.h"
+
+#include "allocation.h"
+#include "common.h"
+#include "cpuinfo.h"
+#include "log.h"
+#include "os_common.h"
+#include "perf_monitoring.h"
+#include "resctrl.h"
+#include "resctrl_alloc.h"
+
 #include <stdlib.h>
 #include <string.h>
 #include <sys/stat.h>
 
-#include "pqos.h"
-#include "allocation.h"
-#include "common.h"
-#include "os_cap.h"
-#include "types.h"
-#include "log.h"
-#include "resctrl.h"
-#include "resctrl_alloc.h"
-#include "perf_monitoring.h"
-#include "cpuinfo.h"
-#include "os_common.h"
-
 static int mba_ctrl = -1; /**< mba ctrl support */
 
 /**
@@ -314,6 +313,7 @@ os_cap_mon_perf_support(const enum pqos_mon_event event,
          * Assume support of perf events
          */
         case PQOS_PERF_EVENT_LLC_MISS:
+        case PQOS_PERF_EVENT_LLC_REF:
         case PQOS_PERF_EVENT_IPC:
                 *supported = 1;
                 return PQOS_RETVAL_OK;
@@ -412,6 +412,7 @@ os_cap_mon_discover(struct pqos_cap_mon **r_cap, const struct pqos_cpuinfo *cpu)
                 PQOS_MON_EVENT_TMEM_BW,
                 PQOS_MON_EVENT_RMEM_BW,
                 PQOS_PERF_EVENT_LLC_MISS,
+                PQOS_PERF_EVENT_LLC_REF,
                 PQOS_PERF_EVENT_IPC
             /* clang-format on */
         };
@@ -441,7 +442,6 @@ os_cap_mon_discover(struct pqos_cap_mon **r_cap, const struct pqos_cpuinfo *cpu)
         cap->l3_size = cpu->l3.total_size;
 
         for (i = 0; i < DIM(events); i++) {
-                int supported;
                 uint32_t scale;
                 struct pqos_cap_mon *mon;
                 struct pqos_monitor *monitor;
@@ -545,7 +545,6 @@ os_cap_l2ca_discover(struct pqos_cap_l2ca *cap, const struct pqos_cpuinfo *cpu)
         cap->mem_size = sizeof(*cap);
         cap->cdp = cdp_on;
         cap->cdp_on = cdp_on;
-        cap->way_size = cpu->l2.way_size;
 
         ret = resctrl_alloc_get_num_closids(&cap->num_classes);
         if (ret != PQOS_RETVAL_OK)
@@ -559,6 +558,8 @@ os_cap_l2ca_discover(struct pqos_cap_l2ca *cap, const struct pqos_cpuinfo *cpu)
         if (ret != PQOS_RETVAL_OK)
                 return ret;
 
+        cap->way_size = cpu->l2.way_size * cpu->l2.num_ways / cap->num_ways;
+
         if (!cdp_on)
                 ret = pqos_file_contains(PROC_CPUINFO, "cdp_l2", &cap->cdp);
 
