@@ -40,39 +40,21 @@ import ctypes
 
 from pqos.capability import pqos_get_type_enum
 from pqos.common import pqos_handle_error, free_memory
+from pqos.native_struct import PqosChannelT
 from pqos.pqos import Pqos
 
 
-def _get_technology_mask(technologies):
-    mask = 0
-    for technology in technologies:
-        tech_mask = 1 << pqos_get_type_enum(technology)
-        mask |= tech_mask
-
-    return mask
-
-
-def _get_list_of_cores(cores):
-    core_array_len = len(cores)
-    core_array = (ctypes.c_uint * core_array_len)(*cores)
-    return core_array
-
-
-def _get_list_of_pids(pids):
-    pid_array_len = len(pids)
-    pid_array = (ctypes.c_uint * pid_array_len)(*pids)
-    return pid_array
-
-
-def _get_cdp_config(cdp_config):
-    cdp_config_map = {
-        'off': 0,
-        'on': 1,
-        'any': 2
+def _get_feature_config(feature_cfg):
+    feature_cfg_map = {
+        'any': 0,
+        'off': 1,
+        'on': 2
     }
 
-    return cdp_config_map[cdp_config.lower()]
+    return feature_cfg_map[feature_cfg.lower()]
 
+def _get_cdp_config(cdp_config):
+    return _get_feature_config(cdp_config)
 
 def _get_mba_config(mba_config):
     mba_config_map = {
@@ -83,6 +65,74 @@ def _get_mba_config(mba_config):
 
     return mba_config_map[mba_config.lower()]
 
+class CPqosAllocConfig(ctypes.Structure):
+    "pqos_alloc_config structure"
+    # pylint: disable=attribute-defined-outside-init
+
+    _fields_ = [
+        ("l3_cdp", ctypes.c_int),
+        ("l2_cdp", ctypes.c_int),
+        ("mba", ctypes.c_int),
+        ("mba40", ctypes.c_int)
+    ]
+
+    def set_l3_cdp(self, l3_cdp):
+        """
+        Sets L3 CDP configuration.
+
+        Parameter:
+            l3_cdp: L3 CDP configuration ('any', 'on' or 'off')
+        """
+
+        self.l3_cdp = _get_cdp_config(l3_cdp)
+
+    def set_l2_cdp(self, l2_cdp):
+        """
+        Sets L2 CDP configuration.
+
+        Parameter:
+            l2_cdp: L2 CDP configuration ('any', 'on' or 'off')
+        """
+
+        self.l2_cdp = _get_cdp_config(l2_cdp)
+
+    def set_mba(self, mba):
+        """
+        Sets MBA configuration.
+
+        Parameter:
+            mba: MBA configuration ('any', 'ctrl' or 'default')
+        """
+
+        self.mba = _get_mba_config(mba)
+
+    def set_mba40(self, mba40):
+        """
+        Sets MBA 4.0 configuration.
+
+        Parameter:
+            mba40: MBA 4.0 configuration ('any', 'on' or 'off')
+        """
+
+        self.mba40 = _get_feature_config(mba40)
+
+def _get_technology_mask(technologies):
+    mask = 0
+    for technology in technologies:
+        tech_mask = 1 << pqos_get_type_enum(technology)
+        mask |= tech_mask
+
+    return mask
+
+def _get_list_of_cores(cores):
+    core_array_len = len(cores)
+    core_array = (ctypes.c_uint * core_array_len)(*cores)
+    return core_array
+
+def _get_list_of_pids(pids):
+    pid_array_len = len(pids)
+    pid_array = (ctypes.c_uint * pid_array_len)(*pids)
+    return pid_array
 
 class PqosAlloc(object):
     """
@@ -290,6 +340,7 @@ class PqosAlloc(object):
                                              mba_cfg_enum)
         pqos_handle_error('pqos_alloc_reset', ret)
 
+
     def reset_config(self, cfg):
         """
         Resets configuration of allocation technologies.
@@ -301,3 +352,81 @@ class PqosAlloc(object):
         cfg_ptr = ctypes.pointer(cfg)
         ret = self.pqos.lib.pqos_alloc_reset_config(cfg_ptr)
         pqos_handle_error('pqos_alloc_reset_config', ret)
+
+    def assoc_get_channel(self, channel):
+        """
+        Gets a class of service associated with a given channel.
+
+        Parameters:
+            channel: a channel
+
+        Returns:
+            a class of service
+        """
+
+        class_id = ctypes.c_uint(0)
+        ref = ctypes.byref(class_id)
+        argtypes = [PqosChannelT, ctypes.POINTER(ctypes.c_uint)]
+        self.pqos.lib.pqos_alloc_assoc_get_channel.argtypes = argtypes
+        ret = self.pqos.lib.pqos_alloc_assoc_get_channel(channel, ref)
+        pqos_handle_error('pqos_alloc_assoc_get_channel', ret)
+        return class_id.value
+
+    def assoc_get_dev(self, segment, bdf, virtual_channel):
+        """
+        Gets a class of service associated with a given device virtual channel.
+
+        Parameters:
+            segment: device segment/domain
+            bdf: device ID
+            virtual_channel: device virtual channel
+
+        Returns:
+            a class of service
+        """
+
+        class_id = ctypes.c_uint(0)
+        ref = ctypes.byref(class_id)
+        argtypes = [ctypes.c_uint16, ctypes.c_uint16, ctypes.c_uint,
+                    ctypes.POINTER(ctypes.c_uint)]
+        self.pqos.lib.pqos_alloc_assoc_get_dev.argtypes = argtypes
+        ret = self.pqos.lib.pqos_alloc_assoc_get_dev(segment, bdf,
+                                                     virtual_channel, ref)
+        pqos_handle_error('pqos_alloc_assoc_get_dev', ret)
+        return class_id.value
+
+    def assoc_set_channel(self, channel, class_id):
+        """
+        Associates a channel with a given class of service.
+
+        Parameters:
+            channel: a channel
+            class_id: a class of service
+        """
+
+        argtypes = [PqosChannelT, ctypes.c_uint]
+        self.pqos.lib.pqos_alloc_assoc_set_channel.argtypes = argtypes
+        ret = self.pqos.lib.pqos_alloc_assoc_set_channel(channel, class_id)
+        pqos_handle_error('pqos_alloc_assoc_set_channel', ret)
+
+    def assoc_set_dev(self, segment, bdf, virtual_channel, class_id):
+        """
+        Associates a channel with a given class of service.
+
+        Parameters:
+            segment: device segment/domain
+            bdf: device ID
+            virtual_channel: device virtual channel
+            class_id: a class of service
+        """
+
+        segment = ctypes.c_uint16(segment)
+        bdf = ctypes.c_uint16(bdf)
+        virtual_channel = ctypes.c_uint(virtual_channel)
+        class_id = ctypes.c_uint(class_id)
+        argtypes = [ctypes.c_uint16, ctypes.c_uint16, ctypes.c_uint,
+                    ctypes.c_uint]
+        self.pqos.lib.pqos_alloc_assoc_set_dev.argtypes = argtypes
+        ret = self.pqos.lib.pqos_alloc_assoc_set_dev(segment, bdf,
+                                                     virtual_channel, class_id)
+        pqos_handle_error('pqos_alloc_assoc_set_dev', ret)
