@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2018-2021 Intel Corporation. All rights reserved.
+ * Copyright(c) 2018-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -42,10 +42,11 @@
 extern "C" {
 #endif
 
-#include <limits.h> /**< CHAR_BIT*/
-
+#include "pqos.h"
 #include "types.h"
 
+#include <limits.h> /**< CHAR_BIT*/
+
 #ifndef RESCTRL_PATH
 #define RESCTRL_PATH "/sys/fs/resctrl"
 #endif
@@ -128,7 +129,7 @@ struct resctrl_cpumask {
  * @brief Set lcore bit in cpu mask
  *
  * @param [in] lcore Core number
- * @param [in] cpumask Modified cpu mask
+ * @param [in] mask Modified cpu mask
  */
 PQOS_LOCAL void resctrl_cpumask_set(const unsigned lcore,
                                     struct resctrl_cpumask *mask);
@@ -137,7 +138,7 @@ PQOS_LOCAL void resctrl_cpumask_set(const unsigned lcore,
  * @brief Unset lcore bit in cpu mask
  *
  * @param [in] lcore Core number
- * @param [in] cpumask Modified cpu mask
+ * @param [in] mask Modified cpu mask
  */
 PQOS_LOCAL void resctrl_cpumask_unset(const unsigned lcore,
                                       struct resctrl_cpumask *mask);
@@ -146,7 +147,7 @@ PQOS_LOCAL void resctrl_cpumask_unset(const unsigned lcore,
  * @brief Check if lcore is set in cpu mask
  *
  * @param [in] lcore Core number
- * @param [in] cpumask Cpu mask
+ * @param [in] mask Cpu mask
  *
  * @return Returns 1 when bit corresponding to lcore is set in mask
  * @retval 1 if cpu bit is set in mask
