@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2017-2021 Intel Corporation. All rights reserved.
+ * Copyright(c) 2017-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -31,20 +31,20 @@
  *
  */
 
+#include "resctrl_alloc.h"
+
+#include "allocation.h"
+#include "cap.h"
+#include "common.h"
+#include "log.h"
+#include "resctrl_monitoring.h"
+#include "resctrl_utils.h"
+
+#include <dirent.h>
+#include <errno.h>
 #include <signal.h>
 #include <stdlib.h>
 #include <string.h>
-#include <errno.h>
-#include <dirent.h>
-
-#include "common.h"
-#include "allocation.h"
-#include "log.h"
-#include "types.h"
-#include "cap.h"
-#include "resctrl_alloc.h"
-#include "resctrl_utils.h"
-#include "resctrl_monitoring.h"
 
 /*
  * COS file names on resctrl file system
@@ -591,9 +591,7 @@ resctrl_alloc_assoc_get(const unsigned lcore, unsigned *class_id)
         unsigned grps;
         unsigned i;
         struct resctrl_cpumask mask;
-        const struct pqos_cap *cap;
-
-        _pqos_cap_get(&cap, NULL);
+        const struct pqos_cap *cap = _pqos_get_cap();
 
         ret = resctrl_alloc_get_grps_num(cap, &grps);
         if (ret != PQOS_RETVAL_OK)
@@ -623,9 +621,7 @@ resctrl_alloc_assoc_set_pid(const pid_t task, const unsigned class_id)
 int
 resctrl_alloc_assoc_get_pid(const pid_t task, unsigned *class_id)
 {
-        const struct pqos_cap *cap;
-
-        _pqos_cap_get(&cap, NULL);
+        const struct pqos_cap *cap = _pqos_get_cap();
 
         /* Search tasks files */
         return resctrl_alloc_task_search(class_id, cap, task);
