@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2019-2021 Intel Corporation. All rights reserved.
+ * Copyright(c) 2019-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -31,14 +31,15 @@
  *
  */
 
+#include "resctrl_schemata.h"
+
+#include "cpuinfo.h"
+#include "log.h"
+#include "resctrl_utils.h"
+
+#include <ctype.h>
 #include <stdlib.h>
 #include <string.h>
-#include <ctype.h>
-#include "cpuinfo.h"
-#include "resctrl_schemata.h"
-#include "resctrl_utils.h"
-#include "types.h"
-#include "log.h"
 
 /*
  * @brief Structure to hold parsed schemata
@@ -685,6 +686,7 @@ int
 resctrl_schemata_mba_write(FILE *fd, const struct resctrl_schemata *schemata)
 {
         unsigned i;
+        int separator = 0;
 
         if (schemata->mba == NULL)
                 return PQOS_RETVAL_OK;
@@ -692,10 +694,17 @@ resctrl_schemata_mba_write(FILE *fd, const struct resctrl_schemata *schemata)
         fprintf(fd, "MB:");
         for (i = 0; i < schemata->mbaids_num; i++) {
                 unsigned id = schemata->mbaids[i];
+                unsigned value = schemata->mba[i].mb_max;
 
-                if (i > 0)
+                /* Do not rewrite default value it will be rounded up to
+                 * MBA granularity */
+                if (value == UINT32_MAX)
+                        continue;
+
+                if (separator)
                         fprintf(fd, ";");
-                fprintf(fd, "%u=%u", id, schemata->mba[i].mb_max);
+                separator = 1;
+                fprintf(fd, "%u=%u", id, value);
         }
         fprintf(fd, "\n");
 
