@@ -1,7 +1,7 @@
 /*
  *   BSD LICENSE
  *
- *   Copyright(c) 2016-2021 Intel Corporation. All rights reserved.
+ *   Copyright(c) 2016-2023 Intel Corporation. All rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,6 +30,12 @@
  *   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+#include "rdt.h"
+
+#include "cpu.h"
+#include "mba_sc.h"
+#include "pqos.h"
+
 #include <ctype.h>
 #include <errno.h>
 #include <inttypes.h>
@@ -40,12 +46,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <pqos.h>
-
-#include "mba_sc.h"
-#include "rdt.h"
-#include "cpu.h"
-
 static const struct pqos_cap *m_cap = NULL;
 static const struct pqos_cpuinfo *m_cpu = NULL;
 static const struct pqos_capability *m_cap_l2ca = NULL;
@@ -167,62 +167,6 @@ rdt_cfg_is_valid(const struct rdt_cfg cfg)
         return 0;
 }
 
-/**
- * @brief Returns number of set bits in \a bitmask
- *
- * @param [in] bitmask
- *
- * @return bits count
- */
-static unsigned
-bits_count(uint64_t bitmask)
-{
-        unsigned count = 0;
-
-        for (; bitmask != 0; count++)
-                bitmask &= bitmask - 1;
-
-        return count;
-}
-
-/**
- * @brief Tests if \a bitmask is contiguous
- *
- * @param [in] cat_type string to identify CAT type (L2 or L3) on error
- * @param [in] bitmask bitmask to be tested
- *
- * @return Result
- * @retval 1 on success
- * @retval 0 on failure
- */
-static int
-is_contiguous(const char *cat_type, const uint64_t bitmask)
-{
-        /* check if bitmask is contiguous */
-        unsigned i = 0, j = 0;
-        const unsigned max_idx = (sizeof(bitmask) * CHAR_BIT);
-
-        if (bitmask == 0)
-                return 0;
-
-        for (i = 0; i < max_idx; i++) {
-                if (((1ULL << i) & bitmask) != 0)
-                        j++;
-                else if (j > 0)
-                        break;
-        }
-
-        if (bits_count(bitmask) != j) {
-                fprintf(stderr,
-                        "Allocation: %s CAT mask 0x%llx is not "
-                        "contiguous.\n",
-                        cat_type, (unsigned long long)bitmask);
-                return 0;
-        }
-
-        return 1;
-}
-
 /**
  * @brief Function to get the highest resource ID (socket/cluster)
  *
@@ -314,7 +258,7 @@ str_to_uint64(const char *str, int base, uint64_t *value)
 
         errno = 0;
         val = strtoull(str_start, &str_end, base);
-        if (errno != 0 || str_end == NULL || str_end == str_start)
+        if (errno != 0 || str_end == NULL || *str_end != '\0')
                 return -EINVAL;
         if (val > UINT64_MAX)
                 return -EINVAL;
@@ -329,51 +273,54 @@ str_to_uint64(const char *str, int base, uint64_t *value)
  *        and stores result in \a mask and \a cmask
  *
  * @param [in] cbm CBM string
- * @param [in] force_dual_mask expect two masks separated with ','
- * @param [out] mask common (L2 or L3 non CDP) or data mask (L3 CDP)
- * @param [out] cmask code mask (L3 CDP)
+ * @param [in] cdp two masks separated with ','
+ * @param [out] data_mask common (L2 or L3 non CDP) or data mask (L3 CDP)
+ * @param [out] code_mask code mask (L3 CDP)
  *
  * @return status
  * @retval 0 on success
  * @retval negative on error (-errno)
  */
 static int
-parse_mask_set(const char *cbm,
-               const int force_dual_mask,
-               uint64_t *mask,
-               uint64_t *cmask)
+parse_mask_set(char *cbm, int *cdp, uint64_t *data_mask, uint64_t *code_mask)
 {
-        int offset = 0;
-        const char *cbm_start = cbm;
+        char *p = cbm;
+        char *saveptr = NULL;
+        uint64_t mask[2] = {0};
+        unsigned mask_count = 0;
 
-        /* parse mask_set from start point */
-        if (*cbm_start == '(' || force_dual_mask) {
-                while (!isxdigit(*cbm_start))
-                        cbm_start++;
+        /* skip brackets */
+        if (*p == '(') {
+                char *q = strchr(p, ')');
 
-                offset = str_to_uint64(cbm_start, 16, cmask);
-                if (offset < 0)
-                        goto err;
-
-                cbm_start += offset;
-
-                while (isblank(*cbm_start))
-                        cbm_start++;
-
-                if (*cbm_start != ',')
-                        goto err;
-
-                cbm_start++;
+                if (q == NULL)
+                        return -EINVAL;
+                ++p;
+                *q = '\0';
         }
 
-        offset = str_to_uint64(cbm_start, 16, mask);
-        if (offset < 0)
-                goto err;
+        for (;; p = NULL) {
+                int ret;
+                char *token = NULL;
 
-        return cbm_start + offset - cbm;
+                token = strtok_r(p, ",", &saveptr);
+                if (token == NULL)
+                        break;
 
-err:
-        return -EINVAL;
+                /* more than 2 masks */
+                if (mask_count > 1)
+                        return -EINVAL;
+
+                ret = str_to_uint64(token, 16, &mask[mask_count++]);
+                if (ret < 0)
+                        return -EINVAL;
+        }
+
+        *data_mask = mask[0];
+        *code_mask = mask[1];
+        *cdp = mask_count > 1;
+
+        return 0;
 }
 
 /**
@@ -388,7 +335,9 @@ print_core_association(const unsigned int cos_id,
                        const unsigned int core_num,
                        const unsigned int *core_array)
 {
-        for (unsigned int i = 0; i < core_num; i++)
+        unsigned i;
+
+        for (i = 0; i < core_num; i++)
                 printf("Core %u => COS%u\n", core_array[i], cos_id);
 }
 
@@ -404,7 +353,9 @@ print_pid_association(const unsigned int cos_id,
                       const unsigned int pid_num,
                       const pid_t *pid_array)
 {
-        for (unsigned int i = 0; i < pid_num; i++)
+        unsigned i;
+
+        for (i = 0; i < pid_num; i++)
                 printf("PID %i => COS%u\n", pid_array[i], cos_id);
 }
 
@@ -432,35 +383,35 @@ parse_reset(const char *cpustr)
  * @retval negative on error (-errno)
  */
 static int
-rdt_ca_str_to_cbm(const char *param, struct rdt_cfg ca)
+rdt_ca_str_to_cbm(char *param, struct rdt_cfg ca)
 {
         uint64_t mask = 0, mask2 = 0;
-        int ret, force_dual_mask;
+        int cdp;
+        int ret;
 
         if (!(PQOS_CAP_TYPE_L2CA == ca.type || PQOS_CAP_TYPE_L3CA == ca.type) ||
             NULL == ca.u.generic_ptr || NULL == param)
                 return -EINVAL;
 
-        force_dual_mask = (NULL != strchr(param, ','));
-        ret = parse_mask_set(param, force_dual_mask, &mask, &mask2);
+        ret = parse_mask_set(param, &cdp, &mask, &mask2);
         if (ret < 0)
                 return -EINVAL;
 
-        if (mask == 0 || is_contiguous(rdt_cfg_get_type_str(ca), mask) == 0)
+        if (mask == 0)
                 return -EINVAL;
 
-        if (mask2 != 0 && is_contiguous(rdt_cfg_get_type_str(ca), mask2) == 0)
+        if (cdp && mask2 == 0)
                 return -EINVAL;
 
         if (PQOS_CAP_TYPE_L2CA == ca.type) {
-                if (mask2 != 0) {
+                if (cdp) {
                         ca.u.l2->cdp = 1;
                         ca.u.l2->u.s.data_mask = mask;
                         ca.u.l2->u.s.code_mask = mask2;
                 } else
                         ca.u.l2->u.ways_mask = mask;
         } else {
-                if (mask2 != 0) {
+                if (cdp) {
                         ca.u.l3->cdp = 1;
                         ca.u.l3->u.s.data_mask = mask;
                         ca.u.l3->u.s.code_mask = mask2;
@@ -602,7 +553,7 @@ parse_rdt(char *rdtstr)
                 }
 
                 const char *feature = strtok_r(group, "=", &group_saveptr);
-                const char *param = strtok_r(NULL, "=", &group_saveptr);
+                char *param = strtok_r(NULL, "=", &group_saveptr);
 
                 if (feature == NULL || param == NULL) {
                         fprintf(stderr, "Invalid option: \"%s\"\n", group);
@@ -1459,9 +1410,15 @@ cfg_configure_cos(const struct pqos_l2ca *l2ca,
                 ca.class_id = cos_id;
 
                 ret = pqos_l3ca_set(l3cat_id, 1, &ca);
-                if (ret != PQOS_RETVAL_OK) {
+                if (ret == PQOS_RETVAL_PARAM) {
                         fprintf(stderr,
-                                "Error setting L3 CAT COS#%u on l3cat id %u!\n",
+                                "Invalid parameters for L3 CAT COS#%u "
+                                "on L3ID %u!\n",
+                                cos_id, l3cat_id);
+                        return -EINVAL;
+                } else if (ret != PQOS_RETVAL_OK) {
+                        fprintf(stderr,
+                                "Error setting L3 CAT COS#%u on L3ID %u!\n",
                                 cos_id, l3cat_id);
                         return -EFAULT;
                 }
@@ -1497,12 +1454,20 @@ cfg_configure_cos(const struct pqos_l2ca *l2ca,
                 /* set proper COS id */
                 ca.class_id = cos_id;
 
-                if (pqos_l2ca_set(l2_id, 1, &ca) != PQOS_RETVAL_OK) {
+                ret = pqos_l2ca_set(l2_id, 1, &ca);
+                if (ret == PQOS_RETVAL_PARAM) {
+                        fprintf(stderr,
+                                "Invalid parameters for L2 CAT COS#%u "
+                                "on L2ID %u!\n",
+                                cos_id, l2_id);
+                        return -EINVAL;
+                } else if (ret != PQOS_RETVAL_OK) {
                         fprintf(stderr,
                                 "Error setting L2 CAT COS#%u on L2ID %u!\n",
                                 cos_id, l2_id);
                         return -EFAULT;
                 }
+
                 if (g_cfg.verbose) {
                         if (ca.cdp)
                                 printf("L2ID %u L2CA COS%u => DATA 0x%lx"
