@@ -1,7 +1,7 @@
 ################################################################################
 # BSD LICENSE
 #
-# Copyright(c) 2019-2022 Intel Corporation. All rights reserved.
+# Copyright(c) 2019-2023 Intel Corporation. All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions
@@ -33,6 +33,7 @@
 import os
 import logging
 import subprocess
+import time
 import psutil
 from testlib.env import Env
 
@@ -69,13 +70,23 @@ class Test:
     def fini(self):
         subprocess.call("killall -9 memtester".split(),
                         stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
+        subprocess.call("killall -9 membw".split(),
+                stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
 
 
     ## Runs command and adds output to log
     def run(self, command, quiet=False):
-        with subprocess.Popen(command.split(), stdin=subprocess.PIPE, stdout=subprocess.PIPE,
-                              stderr=subprocess.PIPE, encoding="utf-8") as child:
-            stdout, stderr = child.communicate()
+        with subprocess.Popen(command.split(), stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+                              encoding="utf-8") as child:
+            (stdout, stderr) = child.communicate()
+
+        # repeat command on RMID exhaustion
+        if "Failed to create resctrl group /sys/fs/resctrl/COS" in stdout:
+            # wait for limbo list cleanup
+            time.sleep(1)
+            with subprocess.Popen(command.split(), stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+                              encoding="utf-8") as child:
+                (stdout, stderr) = child.communicate()
 
         if not quiet:
             self.log.debug(command)
