@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2020-2021 Intel Corporation. All rights reserved.
+ * Copyright(c) 2020-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -30,15 +30,9 @@
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include <string.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <stddef.h>
-#include <setjmp.h>
-#include <cmocka.h>
-
 #include "allocation.h"
 #include "cpu_registers.h"
+#include "test.h"
 
 /* ======== hw_alloc_assoc_read ======== */
 
@@ -51,9 +45,9 @@ test_hw_alloc_assoc_read(void **state __attribute__((unused)))
 
         expect_value(__wrap_msr_read, lcore, lcore);
         expect_value(__wrap_msr_read, reg, PQOS_MSR_ASSOC);
+        will_return(__wrap_msr_read, PQOS_RETVAL_OK);
         will_return(__wrap_msr_read, ((uint64_t)2)
                                          << PQOS_MSR_ASSOC_QECOS_SHIFT);
-        will_return(__wrap_msr_read, PQOS_RETVAL_OK);
 
         ret = hw_alloc_assoc_read(lcore, &class_id);
         assert_int_equal(ret, PQOS_RETVAL_OK);
@@ -79,8 +73,6 @@ test_hw_alloc_assoc_read_error(void **state __attribute__((unused)))
 
         expect_value(__wrap_msr_read, lcore, lcore);
         expect_value(__wrap_msr_read, reg, PQOS_MSR_ASSOC);
-        will_return(__wrap_msr_read, ((uint64_t)2)
-                                         << PQOS_MSR_ASSOC_QECOS_SHIFT);
         will_return(__wrap_msr_read, PQOS_RETVAL_ERROR);
 
         ret = hw_alloc_assoc_read(lcore, &class_id);
@@ -101,4 +93,4 @@ main(void)
         result += cmocka_run_group_tests(tests, NULL, NULL);
 
         return result;
-}
\ No newline at end of file
+}
