@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2020-2021 Intel Corporation. All rights reserved.
+ * Copyright(c) 2020-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -30,19 +30,11 @@
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include <string.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <stddef.h>
-#include <setjmp.h>
-#include <cmocka.h>
-
-#include "test.h"
+#include "allocation.h"
+#include "cpu_registers.h"
 #include "mock_cap.h"
 #include "mock_machine.h"
-
-#include "cpu_registers.h"
-#include "allocation.h"
+#include "test.h"
 
 /* ======== mock ======== */
 
@@ -56,6 +48,7 @@ hw_alloc_assoc_unused(const unsigned technology,
                       unsigned *class_id)
 {
         int ret;
+
         check_expected(technology);
         check_expected(l3cat_id);
         check_expected(l2cat_id);
@@ -103,8 +96,8 @@ test_hw_l3ca_get_min_cbm_bits(void **state)
         unsigned min_cbm_bits;
         unsigned expected_min_cbm_bits = 3;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         expect_value(hw_alloc_assoc_unused, technology, technology);
         expect_value(hw_alloc_assoc_unused, l3cat_id, l3cat_id);
@@ -138,8 +131,8 @@ test_hw_l3ca_get_min_cbm_bits_no_free_cos(void **state)
         unsigned min_cbm_bits;
         unsigned i;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         for (i = 0; i < 2; ++i) {
                 expect_value(hw_alloc_assoc_unused, technology, technology);
@@ -165,4 +158,4 @@ main(void)
         result += cmocka_run_group_tests(tests, test_init_l3ca, test_fini);
 
         return result;
-}
\ No newline at end of file
+}
