@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2020-2021 Intel Corporation. All rights reserved.
+ * Copyright(c) 2020-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -30,21 +30,13 @@
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include <string.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <stddef.h>
-#include <setjmp.h>
-#include <cmocka.h>
-
-#include "test.h"
+#include "allocation.h"
 #include "mock_cap.h"
 #include "mock_resctrl.h"
 #include "mock_resctrl_schemata.h"
-
-#include "allocation.h"
 #include "os_allocation.h"
 #include "resctrl_alloc.h"
+#include "test.h"
 
 /* ======== os_alloc_reset_cores ======== */
 
@@ -55,7 +47,8 @@ test_os_alloc_reset_cores(void **state)
         int ret;
         unsigned i;
 
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         expect_value(__wrap_resctrl_alloc_cpumask_read, class_id, 0);
         will_return(__wrap_resctrl_alloc_cpumask_read, PQOS_RETVAL_OK);
@@ -85,8 +78,8 @@ test_os_alloc_reset_schematas(void **state)
 
         resctrl_alloc_get_grps_num(data->cap, &grps);
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         will_return(__wrap_resctrl_lock_exclusive, PQOS_RETVAL_OK);
         will_return(__wrap_resctrl_lock_release, PQOS_RETVAL_OK);
@@ -129,4 +122,4 @@ main(void)
                                          test_init_unsupported, test_fini);
 
         return result;
-}
\ No newline at end of file
+}
