@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2020-2021 Intel Corporation. All rights reserved.
+ * Copyright(c) 2020-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -41,6 +41,10 @@ int __wrap_resctrl_cpumask_write(FILE *fd, const struct resctrl_cpumask *mask);
 int __wrap_resctrl_cpumask_read(FILE *fd, struct resctrl_cpumask *mask);
 void __wrap_resctrl_cpumask_set(const unsigned lcore,
                                 struct resctrl_cpumask *mask);
+void __wrap_resctrl_cpumask_unset(const unsigned lcore,
+                                  struct resctrl_cpumask *mask);
+int __wrap_resctrl_cpumask_get(const unsigned lcore,
+                               const struct resctrl_cpumask *mask);
 int __wrap_resctrl_mount(const enum pqos_cdp_config l3_cdp_cfg,
                          const enum pqos_cdp_config l2_cdp_cfg,
                          const enum pqos_mba_config mba_cfg);
@@ -48,5 +52,6 @@ int __wrap_resctrl_umount(void);
 int __wrap_resctrl_lock_shared(void);
 int __wrap_resctrl_lock_exclusive(void);
 int __wrap_resctrl_lock_release(void);
+int __wrap_resctrl_is_supported(void);
 
 #endif /* MOCK_RESCTRL_H_ */
